import React, { useEffect, useCallback, useMemo, useState } from 'react'
import { Input, InputNumber, DatePicker, Select } from 'antd'
import locale from 'antd/lib/date-picker/locale/zh_CN'
import moment from 'moment'
import { debounce } from '../../../../../../utils'
import { formAutomaticComputation } from '../../../../../../apis/process'
import UserSelect from '../../../../../widgets/business/PersonSelector/components/UserSelect'
import AccountSelectorCell from './AccountSelectorCell'
import { getUserSelector, GetFieldValueByTableName } from '../../../../../../apis/process'
import eventEmitter from '../../../../../../utils/eventEmitter'
const initUserInfo = {
  fullName: '【本人姓名】',
  Phone: '【本人电话】',
  depart: { name: '【本人部门】' }
}
const loaclPaths = ['【本人姓名】', '【本人电话】', '【本人部门】']
const formAutomatic = async (parmas, callback) => {
  const { code, data } = await formAutomaticComputation(parmas)
  if (code === 0) {
    let otherRow = {}
    let datas = [...parmas.values, ...data]
    datas.forEach(v => {
      otherRow[v.fieldName] = v.fieldValue + ''
    })
    callback(otherRow)
  }
}

const formAuto = debounce(formAutomatic)

const ValueEdit = (props) => {
  const [pullDown, setPullDown] = useState([])
  const preview = sessionStorage.getItem('FormRender')
  const userInfo = preview === 'preview' ? initUserInfo : window?.globalConfig?.userInfo || initUserInfo
  const {
    onChange,
    value,
    fieldName,
    record,
    fieldTableKey,
    autoArray,
    otherChange,
    presetValue,
    disabled,
    placeholder,
    widget,
    isMultiple,
    sourceType,
    options,
    format,
    min,
    max,
    formatter,
    isStoreID,
    displayName,
    tableName,
    whereField,
    whereType,
    _fieldName
  } = props
  const property = isStoreID ? 'userID' : 'userName'
  useEffect(() => {
    console.log('ValueEdit', props)
    switch (widget) {
      case 'TextInput':
      dealText();
      break;
      case 'PersonSelector':
      getUsers();
      break;
      case 'RelevanceSelect':
      dealSelect();
      break;
      default:
      break;
    }
    return () => {
      if(fieldTableKey){
        eventEmitter.removeAllListeners(`${record.ID}_event`);
      }
    }
  }, [])


  const dealSelect = ()=>{
    setTimeout(()=>{
      getTableData();
    }, 100)
    
      if(fieldTableKey){
        eventEmitter.on(`${record.ID}_event`, (val)=> dealParentData(val))
      }
  }

  const dealText = () =>{
    let value = presetValue
    if (loaclPaths.includes(value)) {
      if (value === '【本人姓名】') {
        if (isStoreID) {
          value = userInfo.OID + ''
        } else {
          value = userInfo.fullName
        }
      } else if (value === '【本人部门】') {
        if (isStoreID) {
          value = userInfo.depart.OID + ''
        } else {
          value = userInfo.depart.name
        }
      }
      onChange({ fieldName, fieldValue: value })
    }
  }


  const dealParentData = async (val) => {
    if (!whereField) {
      console.log('请选择联动字段!')
       return
     }
     if (!whereType) {
      console.log('请选择联动条件!')
       return
     }
    const { code, data, msg } = await GetFieldValueByTableName({
      params: {
        tableName: tableName,
        fieldName: `${displayName},${_fieldName}`,
      },
      data: [{
        field: whereField,
        type: whereType,
        value: val
      }]
    })
    if (code === 0) {
      if (Array.isArray(data)) {
        setPullDown(data.map(v => {
          return {
            label: v.find(x=> x.fieldName === displayName)?.fieldValue,
            value:  v.find(x=> x.fieldName === _fieldName)?.fieldValue,
          }
         }))
      } else {
        setPullDown([])
      }
    } else {
      message.error(msg)
    }
  }

  const getTableData = async () => {
    
    if (!tableName) {
      console.log('请选择表名!')
       return
     }
     if (!_fieldName) {
      console.log('请选择存储字段名!')
       return
     }
     if (!displayName) {
      console.log('请选择展示字段名!')
       return
     }
     const res = await GetFieldValueByTableName({
       params: {
         tableName: tableName,
         fieldName: `${displayName},${_fieldName}`,
         time: new Date().getTime()
       },
       data: []
     })
     const { code, data, msg } = res;
     if (code === 0) {
       if (Array.isArray(data)) {
        setPullDown(data.map(v => {
          return {
            label: v.find(x=> x.fieldName === displayName)?.fieldValue,
            value:  v.find(x=> x.fieldName === _fieldName)?.fieldValue,
          }
         }))
       } else {
        setPullDown([])
       }
     } else {
       message.error(msg)
     }
 }

  const getUsers = async () => {
    let time = new Date().getTime() + Math.random()
    const { code, data, msg } = await getUserSelector('', false ,time)
    if (code === 0) {
      if (data) {
        setPullDown(data.dropdownList.map((v, i) => {
          let length = v.groupName?.length
          return {
            label: (
              <div style={{ display: 'flex', justifyContent: 'space-between' }}>
                <span>{v.userName}</span>
                <span style={{ fontSize: '12px', color: '#b4b1b1', }}>{length > 20 ? `...${v.groupName.slice(length - 20, length)}` : v.groupName}</span>
              </div>
            ),
            userName: v.userName,
            value: v[property],
            groupName: v.groupName
          }
        }))
      } else {
        setPullDown([])
      }
    } else {
      message.info(msg)
    }
  }

  if (widget === 'AccountSelector') {
    return <AccountSelectorCell {...props} />
  }
  const renderWidget = (w) => {
    if (['CheckBox', 'ComboBox', 'RadioButton'].includes(w) && sourceType === '手动输入') {
      return (
        <Select
          value={value}
          disabled={disabled}
          style={{ width: '100%' }}
          mode={isMultiple ? 'multiple' : ''}
          onChange={(value) => onChange({ fieldName, fieldValue: value })}
          placeholder={`请输入${fieldName}`}
          options={options?.map(v => ({ label: v.value, value: v.value }))}
          allowClear
        />
      )
    }else if (w === 'DateTime') {
    if(options === '默认为当前时间' && !value){
        onChange({ fieldName, fieldValue: moment().format('YYYY-MM-DD HH:mm:ss') })
    }
      const isValid = value ? moment(value)._isValid : false
      const timeOtions = format === 'dateTime' ?
        {
          showTime: {
            minuteStep: 5,
            format: 'HH:mm'
          },
          format: 'YYYY-MM-DD HH:mm'
        }
        :
        {
          picker: format,
          showTime: false
        }
      return (
        <DatePicker
          disabled={disabled}
          placeholder={disabled ? (placeholder || '') : (placeholder || '点击选择日期')}
          value={isValid ? moment(value) : options === '默认为当前时间' ? moment() : null}
          showNow
          {...timeOtions}
          style={{ width: '100%' }}
          locale={locale}
          onChange={(date) => {
            if (date && date._d) {
              onChange({ fieldName, fieldValue: moment(date._d).format('YYYY-MM-DD HH:mm:ss') })
            } else {
              onChange({ fieldName, fieldValue: '' })
            }
          }}
        />
      )
    }else if (w === 'NumberInput') {
      return (
        <InputNumber
          min={min || Number.MIN_SAFE_INTEGER}
          max={max || Number.MAX_SAFE_INTEGER}
          step={(formatter === '${百分比}' && isStoreFormatter) ? 0.01 : 1}
          placeholder={disabled ? (placeholder || '') : (placeholder || '请输入内容')}
          disabled={disabled}
          value={value === '' ? null : Number(value)}
          onChange={(value) => {
            onChange({ fieldName, fieldValue: value !== null ? `${value}` : '' })
            if (autoArray.length) {
              autoArray.forEach(v => {
                if (v.fields.includes(fieldName)) {
                  let values = v.fields.map(s => {
                    if (s === fieldName) {
                      return { fieldName, fieldValue: value !== null ? `${value}` : '' }
                    }
                    return { fieldName: s, fieldValue: record?.[s] || '' }
                  })
                  formAuto({ ...v, values }, otherChange)
                }
              })
            }
          }}
          style={{ width: '100%' }}
        />
      )
    }else if(w === 'PersonSelector'){
      return (
        <Select
          value={value}
          style={{ width: '100%' }}
          disabled={disabled}
          showSearch
          mode={isMultiple ? 'multiple' : ''}
          onChange={(value) => onChange({ fieldName, fieldValue: value })}
          placeholder={`请输入${fieldName}`}
          options={pullDown}
          filterOption={(input, option) => (option?.userName || '').toLowerCase().includes(input.toLowerCase())}
          allowClear
        />
      )
    }else if(w === 'RelevanceSelect'){
      return (
        <Select
          value={value}
          style={{ width: '100%' }}
          disabled={disabled}
          onChange={(value) => {
            onChange({ fieldName, fieldValue: value })
            if(!fieldTableKey){
              eventEmitter.emit(`${record.ID}_event`, value)
            }
          }}
          placeholder={`请输入${_fieldName}`}
          options={pullDown}
          allowClear
        />
      )
    }else{
      return (
        <Input
          disabled={disabled}
          style={{ color: loaclPaths.includes(presetValue) ? 'green' : '' }}
          onChange={(e) => onChange({ fieldName, fieldValue: e.target.value })}
          value={value}
        />
      )
    }
  }

  

  return (<>{ renderWidget(widget) }</>)

}

export default ValueEdit